#include "Headers/Registry.H"
#include "Headers/Interface.H"
#include "Headers/LZARI.H"

//////////////////////////////////////////////////////////////////////

void DisplayHelp(void)
{
	printf("\n%s %s\n",TitleCaption,GammaVersion);
	printf("\n 448Cmd.Exe [Input] [Output] [Password] [Option]\n");
	printf("---------------------------------------------\n");
	printf("   [Input] : Name of file to be encrypted or directory to pack.\n");
	printf("  [Output] : Name of encrypted file to be written to.\n");
	printf("[Password] : Key to encrypt the file with.\n");
	printf(" [Options] : /C = Compress /Exe = Self Extractor\n");
}

//////////////////////////////////////////////////////////////////////

void Error(char *xMessage)
{
	printf("Error: %s\n",xMessage);
	exit(0);
}

//////////////////////////////////////////////////////////////////////

int PreCheckFile(char *FileToCheck)
{
	FILE * FileToCheck_File;

	unsigned int Attrib = 0;
	char xDrive[255], xDir[255], xFile[255], xExt[255];
	_splitpath(FileToCheck,xDrive,xDir,xFile,xExt);

	/* Get Input File Attributes */
	Attrib = GetFileAttributes(FileToCheck);

	/* Check For File Access */
	if(!FileAccess(FileToCheck,0)) // Check Exist
		Error("File Does Not Exist. Error 9.");

	if(!FileAccess(FileToCheck,4)) // Check Read Access
		Error("File Does Not Allow Read Access. Error 11.");

	if(Attrib & FILE_ATTRIBUTE_DIRECTORY) // If Input Is A Folder
	{
		return 1;
	}

	DWORD FileSize_Temp = Get_FileSize(FileToCheck);
	if(FileSize_Temp == -1)return 0;

	if(FileSize_Temp > MaxFileSize*MegaByte)
	{
		char MessageTemp[257];
		sprintf(MessageTemp,"448Delta Console %s Requires That Files Be Under %d MB",ShortVersion,MaxFileSize);
		printf(MessageTemp);
		return 0;
	}

	if(FileSize_Temp < 1)
	{
		char MessageTemp[257];
		sprintf(MessageTemp,"448Delta Console %s Requires That Files Contain Some Data.",ShortVersion);
		printf(MessageTemp);
		return 0;
	}

	/* Open Source File */
	if((FileToCheck_File = fopen(FileToCheck,"rb")) == NULL)
		Error("Could Not Open Source For Read. Error: 13.");

	/* Read File Flag ( If Any ) */
	fread(Buffer,sizeof(char),FlagLEN,FileToCheck_File);

	if(strcmp(Buffer,ENCFlag) == 0) // If File Encryted
	{
		if(FileToCheck_File)
			fclose(FileToCheck_File);
		return 2;
	}

	if(strcmp(Buffer,PACFlag) == 0) // If File Packed
	{
		if(FileToCheck_File)
			fclose(FileToCheck_File);
		return 3;
	}

	if(strcmp(Buffer,ENCCompress) == 0) // If File Compressed Encryted
	{
		if(FileToCheck_File)
			fclose(FileToCheck_File);
		return 4;
	}

	if(strcmp(Buffer,PACCompress) == 0) // If File Compressed Packed
	{
		if(FileToCheck_File)
			fclose(FileToCheck_File);
		return 5;
	}

	if(FileToCheck_File)
		fclose(FileToCheck_File);
	return 6; // If File Not Encrypted
}

//////////////////////////////////////////////////////////////////////

int ProcessInput(char *InputName, char *OutputName, char *Password)
{
	short ENCMethod = 0;

	sprintf(GlobalFileName,"%s",InputName);

	printf("%s %s\n\n",TitleCaption,ShortVersion);

	if(strlen(Password) < 8)
	{
		printf("Key To Short... Must be at least 8 characters.\n");
		return 0;
	}

	if(strlen(Password) > 56)
	{
		printf("Key To Long... Must no longer than 56 characters.\n");
		return 0;
	}

	printf("Password Strength = %d-Bits\n",strlen(Password)*8);

	/*
	0 = Cancel , Dir = 1 , Encrypted = 2 , Packed = 3
	Compressed Encrypted = 4 , Compressed Packed = 5
	Not Encrypted = 6
	*/

	ENCMethod = PreCheckFile(GlobalFileName);

	if(ENCMethod == 0) // Cancel
	{
		return(0);
	}

	if(ENCMethod == 1) // Directory
	{
		if(UseCompression)
		{
			char CompressTemp[257];
			char PackTemp[257];

			CompressPack = TRUE;

			printf("Compress/Packing %s\n",InputName);

			sprintf(CompressTemp,"%s\\%s",Get_TempDirectory(),"Compress.Tmp");
			sprintf(PackTemp,"%s\\%s",Get_TempDirectory(),"PackFiles.Tmp");

			printf("\n");
			if(!PackFiles(InputName,PackTemp))return 0;
			printf("\n");
			if(!LZARICompress(PackTemp,CompressTemp))return 0;
			printf("\n");
			if(!BlowFish_Interface(CompressTemp, OutputName, Password, 3))return 0;
			printf("\n");

			_fcloseall();

			unlink(CompressTemp);
			unlink(PackTemp);
			return 1;
		}

		if(!UseCompression)
		{
			char PackTemp[257];

			Pack = TRUE;

			printf("Packing %s\n",InputName);

			sprintf(PackTemp,"%s\\%s",Get_TempDirectory(),"PackFiles.Tmp");

			printf("\n");
			if(!PackFiles(InputName,PackTemp))return 0;
			printf("\n");
			if(!BlowFish_Interface(PackTemp, OutputName, Password, 3))return 0;
			printf("\n");

			_fcloseall();

			unlink(PackTemp);
			return 1;
		}
	}

	if(ENCMethod == 2) // Encrypted
	{
		printf("Decrypting %s, Size %d Bytes.\n",InputName,Get_FileSize(InputName));

		printf("\n");
		if(!BlowFish_Interface(InputName, OutputName, Password, 1))return 0;
		printf("\n");
		return(0);
	}

	if(ENCMethod == 3) //  Packed
	{
		char PackTemp[257];

		printf("UnPacking %s, Size %d Bytes.\n",InputName,Get_FileSize(InputName));
		sprintf(PackTemp,"%s\\%s",Get_TempDirectory(),"PackFiles.Tmp");

		printf("\n");
		if(!BlowFish_Interface(InputName, PackTemp, Password, 1))return 0;
		printf("\n");
		CreateDirectory(OutputName,NULL);
		if(!UnPackFiles(PackTemp,OutputName))return 0;
		printf("\n");

		_fcloseall();
		unlink(PackTemp);
		return 1;
	}

	if(ENCMethod == 4) //  Compressed Encrypted
	{
		char CompressTemp[257];
		sprintf(CompressTemp,"%s\\%s",Get_TempDirectory(),"Compress.Tmp");
		printf("Decompress/Decrypting %s, Size %d Bytes.\n",InputName,Get_FileSize(InputName));

		printf("\n");
		if(!BlowFish_Interface(InputName, CompressTemp, Password, 1))return 0;
		printf("\n");
		if(!LZARIDecompress(CompressTemp,OutputName))return 0;
		printf("\n");

		_fcloseall();
		unlink(CompressTemp);
		return 1;

	}

	if(ENCMethod == 5) //  Compressed Packed
	{
		char PackTemp[257];
		char CompressTemp[257];

		printf("Decompress/UnPacking %s, Size %d Bytes.\n",InputName,Get_FileSize(InputName));
		sprintf(PackTemp,"%s\\%s",Get_TempDirectory(),"PackFiles.Tmp");
		sprintf(CompressTemp,"%s\\%s",Get_TempDirectory(),"Compress.Tmp");

		printf("\n");
		if(!BlowFish_Interface(InputName, CompressTemp, Password, 1))return 0;
		printf("\n");
		if(!LZARIDecompress(CompressTemp,PackTemp))return 0;
		CreateDirectory(OutputName,NULL);
		printf("\n");
		if(!UnPackFiles(PackTemp,OutputName))return 0;

		_fcloseall();
		unlink(PackTemp);
		unlink(CompressTemp);
		return 1;
	}

	if(ENCMethod == 6) // NOT Encrypted
	{
		if(UseCompression)
		{
			char CompressTemp[257];

			CompressEncrypt = TRUE;

			sprintf(CompressTemp,"%s\\%s",Get_TempDirectory(),"Compress.Tmp");
			printf("Compress/Encrypting %s, Size %d Bytes.\n",InputName,Get_FileSize(InputName));

			printf("\n");
			if(!LZARICompress(InputName,CompressTemp))return 0;
			printf("\n");
			if(!BlowFish_Interface(CompressTemp, OutputName, Password, 3))return 0;
			printf("\n");

			_fcloseall();
			unlink(CompressTemp);
			return 1;
		}

		if(!UseCompression)
		{
			Encrypt = TRUE;
			printf("Encrypting %s, Size %d Bytes.\n",InputName,Get_FileSize(InputName));

			printf("\n");
			if(!BlowFish_Interface(InputName, OutputName, Password, 3))return 0;
			printf("\n");
			return 1;
		}
	}

	return(0);
}

//////////////////////////////////////////////////////////////////////

void FreeMemory(void)
{
	while(Package.FolderCount != -1){
		free(Package.FolderNames[Package.FolderCount]);
		Package.FolderNames[Package.FolderCount] = '\0';
		Package.FolderCount--;
	}
	while(Package.FileCount != -1){
		free(Package.FileNames[Package.FileCount]);
		Package.FileNames[Package.FileCount] = '\0';
		Package.FileCount--;
	}
}

//////////////////////////////////////////////////////////////////////

void Clear_ReadTemp(void)
{
	int LocalLoop = 0;
	while(LocalLoop != sizeof(ReadTemp))
	{
		ReadTemp[LocalLoop] = '\0';
		LocalLoop++;
	}
}

//////////////////////////////////////////////////////////////////////

int CreateSelfExtractor(char *xInputFile)
{
	FILE *LocalSource, *LocalTarget;

	int SentSize = 0;

	char Buffer[BufferSize+1];
	char TargetFileName[257];
	char Path[256];
	char Drive[256];
	char Directory[256];
	char File[256];
	char Extention[256];

	DWORD PrintCount = 0;

	long double StatusValue = 0;
	long double SourceFileSize = 0;
	long double Current_Size = 0;

	_splitpath(xInputFile,Drive,Directory,File,Extention);
	_makepath(Path,Drive,Directory,File,NULL);

	sprintf(TargetFileName,"%s.Exe",Path);

	printf("Creating Self Extracting EXE\n",StatusValue);

	if(GetSelfExtractPath())
	{
		return 0;
	}

	// Open Executable Source For Read
	if((LocalSource = fopen(SelfExtractPath,"rb")) == NULL)
	{
		if(LocalSource)fclose(LocalSource);
		Error("Could Not Open File For Read. Error 1.");
		return 0;
	}

	// Open Target (Empty Executable)  For Write
	if((LocalTarget = fopen(TargetFileName,"wb")) == NULL)
	{
		if(LocalTarget)fclose(LocalTarget);
		if(LocalTarget)fclose(LocalTarget);
		Error("Could Not Open File For Write. Error 2.");
		return 0;
	}

	// Write executable to empty file
	do
	{
		Buffer[0] = '\0';
		SentSize = fread(Buffer,sizeof(char),BufferSize,LocalSource);
		fwrite(Buffer,sizeof(char),SentSize,LocalTarget);
	}
	while(SentSize == BufferSize);

	if(LocalSource)fclose(LocalSource);

	// Open 448Delta File For Read
	if((LocalSource = fopen(xInputFile,"rb")) == NULL)
	{
		if(LocalSource)fclose(LocalSource);
		if(LocalTarget)fclose(LocalTarget);
		Error("Could Not Open File For Read. Error 3.");
		return 0;
	}

	SentSize = 0;

	fseek(LocalSource, 0L, SEEK_END);
	SourceFileSize = ftell(LocalSource);
	fseek(LocalSource, 0L, SEEK_SET);

	// Write packed and compressed file to new executable
	do
	{
		Buffer[0] = '\0';
		SentSize = fread(Buffer,sizeof(char),BufferSize,LocalSource);
		fwrite(Buffer,sizeof(char),SentSize,LocalTarget);

		Current_Size += SentSize;
		PrintCount += SentSize;

		if(PrintCount == 2048)
		{
			StatusValue = ((Current_Size / SourceFileSize) * 100);
			printf("Complete: %.1f%%\r",StatusValue);
			PrintCount = 0;
		}

	}
	while(SentSize == BufferSize);

	// Update StatusBar One More Time
	StatusValue = ((Current_Size / SourceFileSize) * 100);
	printf("Complete: %.1f%%\r",StatusValue);
	printf("\n");

	// Close Files
	if(LocalSource)fclose(LocalSource);
	if(LocalTarget)fclose(LocalTarget);
	_fcloseall();

	return 0;
}

/////////////////////////////////////////////////////////////////////

int GetSelfExtractPath()
{
	char GammaPath[1024];
	unsigned long SizeOfGammaPath = sizeof(GammaPath);
	/* Get 448Delta Executable File Name */
	if(GetStringValue(HKEY_CLASSES_ROOT,"*\\Shell\\448Delta_Encryption\\AppInfo\\Path","SelfExtract",GammaPath,SizeOfGammaPath))
	{
		if(FileAccess(GammaPath,0)) // If File Exist
		{
			SelfExtractPath = GammaPath;
			return 0;
		}
		else{ // If File Does Not Exist
			Error("Self EXE Execute Template Not Found.");
			return 1;
		}
	}
	else{ // If Registry Error Or Key Not Found
		Error("Self EXE Registry Key Not Found.");
		return 1;
	}
}

/////////////////////////////////////////////////////////////////////

BOOL CheckExecutable(void)
{

	//--------------(Check Executable Size)---------------------------

	char NameBuffer[1024];
	DWORD SizeofNameBuffer = sizeof(NameBuffer);

	if(GetModuleFileName(NULL,NameBuffer,SizeofNameBuffer) == 0)
	{
		MessageBox(0, "448Cmd Could Not Access The Executable." ,TitleCaption, 0 + MB_TASKMODAL);
		return FALSE;
	}
	else{	 //--------------(Check Executable Size)---------------------------
		if(Get_FileSize(NameBuffer) != ExecutableSize)
		{
			MessageBox(0, "448Cmd Executable Is Invalid." ,TitleCaption, 0 + MB_TASKMODAL);
			return FALSE;
		}
	}

	ShortVersion	= Delta_ShortVersion;

	MaxFiles		= Delta_MaxFiles;
	MaxPackFileSize = Delta_MaxPackFileSize;
	MaxFileSize 	= Delta_MaxFileSize;

	return TRUE;
}

//////////////////////////////////////////////////////////////////////

